@@ -124,3 +124,8 @@ try:
     pass
 except Exception:
     error("...", exc_info=True)
+
+try:
+    ...
+except Exception as e:
+    raise ValueError from e
