@@ -73,3 +73,8 @@ def foo():
 
 async def test():
     return [check async for check in async_func()]
+
+
+async def test() -> str:
+    vals = [str(val) for val in await async_func(1)]
+    return ",".join(vals)
