@@ -83,6 +83,7 @@ pub(crate) fn delete_comment(range: TextRange, locator: &Locator) -> Edit {
     }
     // Ex) `x = 1  # noqa`
     else if range.end() + trailing_space_len == line_range.end() {
+        // Replace `x = 1  # noqa` with `x = 1`.
         Edit::deletion(range.start() - leading_space_len, line_range.end())
     }
     // Ex) `x = 1  # noqa  # type: ignore`
@@ -93,13 +94,15 @@ pub(crate) fn delete_comment(range: TextRange, locator: &Locator) -> Edit {
         ))
         .starts_with('#')
     {
+        // Replace `# noqa  # type: ignore` with `# type: ignore`.
         Edit::deletion(range.start(), range.end() + trailing_space_len)
     }
     // Ex) `x = 1  # noqa here`
     else {
-        Edit::deletion(
-            range.start() + "# ".text_len(),
-            range.end() + trailing_space_len,
+        // Replace `# noqa here` with `# here`.
+        Edit::range_replacement(
+            "# ".to_string(),
+            TextRange::new(range.start(), range.end() + trailing_space_len),
         )
     }
 }
@@ -119,6 +122,28 @@ pub(crate) fn remove_unused_imports<'a>(
     }
 }
 
+/// Edits to make the specified imports explicit, e.g. change `import x` to `import x as x`.
+pub(crate) fn make_redundant_alias<'a>(
+    member_names: impl Iterator<Item = &'a str>,
+    stmt: &Stmt,
+) -> Vec<Edit> {
+    let aliases = match stmt {
+        Stmt::Import(ast::StmtImport { names, .. }) => names,
+        Stmt::ImportFrom(ast::StmtImportFrom { names, .. }) => names,
+        _ => {
+            return Vec::new();
+        }
+    };
+    member_names
+        .filter_map(|name| {
+            aliases
+                .iter()
+                .find(|alias| alias.asname.is_none() && name == alias.name.id)
+                .map(|alias| Edit::range_replacement(format!("{name} as {name}"), alias.range))
+        })
+        .collect()
+}
+
 #[derive(Debug, Copy, Clone)]
 pub(crate) enum Parentheses {
     /// Remove parentheses, if the removed argument is the only argument left.
@@ -454,11 +479,12 @@ fn all_lines_fit(
 mod tests {
     use anyhow::Result;
 
+    use ruff_diagnostics::Edit;
     use ruff_python_parser::parse_suite;
     use ruff_source_file::Locator;
-    use ruff_text_size::{Ranged, TextSize};
+    use ruff_text_size::{Ranged, TextRange, TextSize};
 
-    use crate::fix::edits::{next_stmt_break, trailing_semicolon};
+    use crate::fix::edits::{make_redundant_alias, next_stmt_break, trailing_semicolon};
 
     #[test]
     fn find_semicolon() -> Result<()> {
@@ -529,4 +555,35 @@ x = 1 \
             TextSize::from(12)
         );
     }
+
+    #[test]
+    fn redundant_alias() {
+        let contents = "import x, y as y, z as bees";
+        let program = parse_suite(contents).unwrap();
+        let stmt = program.first().unwrap();
+        assert_eq!(
+            make_redundant_alias(["x"].into_iter(), stmt),
+            vec![Edit::range_replacement(
+                String::from("x as x"),
+                TextRange::new(TextSize::new(7), TextSize::new(8)),
+            )],
+            "make just one item redundant"
+        );
+        assert_eq!(
+            make_redundant_alias(vec!["x", "y"].into_iter(), stmt),
+            vec![Edit::range_replacement(
+                String::from("x as x"),
+                TextRange::new(TextSize::new(7), TextSize::new(8)),
+            )],
+            "the second item is already a redundant alias"
+        );
+        assert_eq!(
+            make_redundant_alias(vec!["x", "z"].into_iter(), stmt),
+            vec![Edit::range_replacement(
+                String::from("x as x"),
+                TextRange::new(TextSize::new(7), TextSize::new(8)),
+            )],
+            "the third item is already aliased to something else"
+        );
+    }
 }
