@@ -1,4 +1,9 @@
+use ruff_python_ast::name::QualifiedName;
 use ruff_python_ast::{self as ast, Expr};
+use ruff_python_semantic::SemanticModel;
+
+use crate::checkers::ast::Checker;
+use crate::settings::LinterSettings;
 
 /// Returns `true` if a function call is allowed to use a boolean trap.
 pub(super) fn is_allowed_func_call(name: &str) -> bool {
@@ -43,6 +48,24 @@ pub(super) fn is_allowed_func_call(name: &str) -> bool {
     )
 }
 
+/// Returns `true` if a call is allowed by the user to use a boolean trap.
+pub(super) fn is_user_allowed_func_call(
+    call: &ast::ExprCall,
+    semantic: &SemanticModel,
+    settings: &LinterSettings,
+) -> bool {
+    semantic
+        .resolve_qualified_name(call.func.as_ref())
+        .is_some_and(|qualified_name| {
+            settings
+                .flake8_boolean_trap
+                .extend_allowed_calls
+                .iter()
+                .map(|target| QualifiedName::from_dotted_name(target))
+                .any(|target| qualified_name == target)
+        })
+}
+
 /// Returns `true` if a function definition is allowed to use a boolean trap.
 pub(super) fn is_allowed_func_def(name: &str) -> bool {
     matches!(name, "__setitem__" | "__post_init__")
@@ -51,7 +74,7 @@ pub(super) fn is_allowed_func_def(name: &str) -> bool {
 /// Returns `true` if an argument is allowed to use a boolean trap. To return
 /// `true`, the function name must be explicitly allowed, and the argument must
 /// be either the first or second argument in the call.
-pub(super) fn allow_boolean_trap(call: &ast::ExprCall) -> bool {
+pub(super) fn allow_boolean_trap(call: &ast::ExprCall, checker: &Checker) -> bool {
     let func_name = match call.func.as_ref() {
         Expr::Attribute(ast::ExprAttribute { attr, .. }) => attr.as_str(),
         Expr::Name(ast::ExprName { id, .. }) => id.as_str(),
@@ -76,5 +99,10 @@ pub(super) fn allow_boolean_trap(call: &ast::ExprCall) -> bool {
         }
     }
 
+    // If the call is explicitly allowed by the user, then the boolean trap is allowed.
+    if is_user_allowed_func_call(call, checker.semantic(), checker.settings) {
+        return true;
+    }
+
     false
 }
