@@ -3,14 +3,13 @@ use std::borrow::Cow;
 use ruff_formatter::{format_args, write, FormatError, FormatOptions, SourceCode};
 use ruff_python_ast::{AnyNodeRef, AstNode, NodeKind, PySourceType};
 use ruff_python_trivia::{
-    is_pragma_comment, lines_after, lines_after_ignoring_trivia, lines_before,
+    is_pragma_comment, lines_after, lines_after_ignoring_trivia, lines_before, CommentLinePosition,
 };
 use ruff_text_size::{Ranged, TextLen, TextRange};
 
-use crate::comments::{CommentLinePosition, SourceComment};
+use crate::comments::SourceComment;
 use crate::context::NodeLevel;
 use crate::prelude::*;
-use crate::preview::is_blank_line_after_nested_stub_class_enabled;
 use crate::statement::suite::should_insert_blank_line_after_class_in_stub_file;
 
 /// Formats the leading comments of a node.
@@ -544,10 +543,7 @@ pub(crate) fn empty_lines_before_trailing_comments<'a>(
     // Black has different rules for stub vs. non-stub and top level vs. indented
     let empty_lines = match (f.options().source_type(), f.context().node_level()) {
         (PySourceType::Stub, NodeLevel::TopLevel(_)) => 1,
-        (PySourceType::Stub, _) => u32::from(
-            is_blank_line_after_nested_stub_class_enabled(f.context())
-                && node_kind == NodeKind::StmtClassDef,
-        ),
+        (PySourceType::Stub, _) => u32::from(node_kind == NodeKind::StmtClassDef),
         (_, NodeLevel::TopLevel(_)) => 2,
         (_, _) => 1,
     };
