@@ -6,12 +6,12 @@ use std::borrow::Cow;
 use std::env::VarError;
 use std::num::{NonZeroU16, NonZeroU8};
 use std::path::{Path, PathBuf};
+use std::str::FromStr;
 
 use anyhow::{anyhow, Result};
 use glob::{glob, GlobError, Paths, PatternError};
 use itertools::Itertools;
 use regex::Regex;
-use ruff_linter::settings::fix_safety_table::FixSafetyTable;
 use rustc_hash::{FxHashMap, FxHashSet};
 use shellexpand;
 use shellexpand::LookupError;
@@ -24,10 +24,11 @@ use ruff_linter::registry::RuleNamespace;
 use ruff_linter::registry::{Rule, RuleSet, INCOMPATIBLE_CODES};
 use ruff_linter::rule_selector::{PreviewOptions, Specificity};
 use ruff_linter::rules::pycodestyle;
+use ruff_linter::settings::fix_safety_table::FixSafetyTable;
 use ruff_linter::settings::rule_table::RuleTable;
 use ruff_linter::settings::types::{
-    ExtensionMapping, FilePattern, FilePatternSet, PerFileIgnore, PerFileIgnores, PreviewMode,
-    PythonVersion, SerializationFormat, UnsafeFixes, Version,
+    CompiledPerFileIgnoreList, ExtensionMapping, FilePattern, FilePatternSet, PerFileIgnore,
+    PreviewMode, PythonVersion, RequiredVersion, SerializationFormat, UnsafeFixes,
 };
 use ruff_linter::settings::{LinterSettings, DEFAULT_SELECTORS, DUMMY_VARIABLE_RGX, TASK_TAGS};
 use ruff_linter::{
@@ -39,10 +40,11 @@ use ruff_python_formatter::{
 };
 
 use crate::options::{
-    Flake8AnnotationsOptions, Flake8BanditOptions, Flake8BugbearOptions, Flake8BuiltinsOptions,
-    Flake8ComprehensionsOptions, Flake8CopyrightOptions, Flake8ErrMsgOptions, Flake8GetTextOptions,
-    Flake8ImplicitStrConcatOptions, Flake8ImportConventionsOptions, Flake8PytestStyleOptions,
-    Flake8QuotesOptions, Flake8SelfOptions, Flake8TidyImportsOptions, Flake8TypeCheckingOptions,
+    Flake8AnnotationsOptions, Flake8BanditOptions, Flake8BooleanTrapOptions, Flake8BugbearOptions,
+    Flake8BuiltinsOptions, Flake8ComprehensionsOptions, Flake8CopyrightOptions,
+    Flake8ErrMsgOptions, Flake8GetTextOptions, Flake8ImplicitStrConcatOptions,
+    Flake8ImportConventionsOptions, Flake8PytestStyleOptions, Flake8QuotesOptions,
+    Flake8SelfOptions, Flake8TidyImportsOptions, Flake8TypeCheckingOptions,
     Flake8UnusedArgumentsOptions, FormatOptions, IsortOptions, LintCommonOptions, LintOptions,
     McCabeOptions, Options, Pep8NamingOptions, PyUpgradeOptions, PycodestyleOptions,
     PydocstyleOptions, PyflakesOptions, PylintOptions,
@@ -116,7 +118,7 @@ pub struct Configuration {
     pub unsafe_fixes: Option<UnsafeFixes>,
     pub output_format: Option<SerializationFormat>,
     pub preview: Option<PreviewMode>,
-    pub required_version: Option<Version>,
+    pub required_version: Option<RequiredVersion>,
     pub extension: Option<ExtensionMapping>,
     pub show_fixes: Option<bool>,
 
@@ -145,10 +147,12 @@ pub struct Configuration {
 impl Configuration {
     pub fn into_settings(self, project_root: &Path) -> Result<Settings> {
         if let Some(required_version) = &self.required_version {
-            if &**required_version != RUFF_PKG_VERSION {
+            let ruff_pkg_version = pep440_rs::Version::from_str(RUFF_PKG_VERSION)
+                .expect("RUFF_PKG_VERSION is not a valid PEP 440 version specifier");
+            if !required_version.contains(&ruff_pkg_version) {
                 return Err(anyhow!(
                     "Required version `{}` does not match the running version `{}`",
-                    &**required_version,
+                    required_version,
                     RUFF_PKG_VERSION
                 ));
             }
@@ -234,6 +238,7 @@ impl Configuration {
                 project_root: project_root.to_path_buf(),
             },
 
+            #[allow(deprecated)]
             linter: LinterSettings {
                 rules: lint.as_rule_table(lint_preview)?,
                 exclude: FilePatternSet::try_from_iter(lint.exclude.unwrap_or_default())?,
@@ -250,11 +255,11 @@ impl Configuration {
                     .dummy_variable_rgx
                     .unwrap_or_else(|| DUMMY_VARIABLE_RGX.clone()),
                 external: lint.external.unwrap_or_default(),
-                ignore_init_module_imports: lint.ignore_init_module_imports.unwrap_or_default(),
+                ignore_init_module_imports: lint.ignore_init_module_imports.unwrap_or(true),
                 line_length,
                 tab_size: self.indent_width.unwrap_or_default(),
                 namespace_packages: self.namespace_packages.unwrap_or_default(),
-                per_file_ignores: PerFileIgnores::resolve(
+                per_file_ignores: CompiledPerFileIgnoreList::resolve(
                     lint.per_file_ignores
                         .unwrap_or_default()
                         .into_iter()
@@ -288,6 +293,10 @@ impl Configuration {
                     .flake8_bandit
                     .map(Flake8BanditOptions::into_settings)
                     .unwrap_or_default(),
+                flake8_boolean_trap: lint
+                    .flake8_boolean_trap
+                    .map(Flake8BooleanTrapOptions::into_settings)
+                    .unwrap_or_default(),
                 flake8_bugbear: lint
                     .flake8_bugbear
                     .map(Flake8BugbearOptions::into_settings)
@@ -605,6 +614,7 @@ pub struct LintConfiguration {
     // Plugins
     pub flake8_annotations: Option<Flake8AnnotationsOptions>,
     pub flake8_bandit: Option<Flake8BanditOptions>,
+    pub flake8_boolean_trap: Option<Flake8BooleanTrapOptions>,
     pub flake8_bugbear: Option<Flake8BugbearOptions>,
     pub flake8_builtins: Option<Flake8BuiltinsOptions>,
     pub flake8_comprehensions: Option<Flake8ComprehensionsOptions>,
@@ -647,6 +657,10 @@ impl LintConfiguration {
             .flatten()
             .chain(options.common.extend_unfixable.into_iter().flatten())
             .collect();
+
+        #[allow(deprecated)]
+        let ignore_init_module_imports = options.common.ignore_init_module_imports;
+
         Ok(LintConfiguration {
             exclude: options.exclude.map(|paths| {
                 paths
@@ -689,7 +703,7 @@ impl LintConfiguration {
                 })
                 .unwrap_or_default(),
             external: options.common.external,
-            ignore_init_module_imports: options.common.ignore_init_module_imports,
+            ignore_init_module_imports,
             explicit_preview_rules: options.common.explicit_preview_rules,
             per_file_ignores: options.common.per_file_ignores.map(|per_file_ignores| {
                 per_file_ignores
@@ -705,6 +719,7 @@ impl LintConfiguration {
             // Plugins
             flake8_annotations: options.common.flake8_annotations,
             flake8_bandit: options.common.flake8_bandit,
+            flake8_boolean_trap: options.common.flake8_boolean_trap,
             flake8_bugbear: options.common.flake8_bugbear,
             flake8_builtins: options.common.flake8_builtins,
             flake8_comprehensions: options.common.flake8_comprehensions,
@@ -852,11 +867,7 @@ impl LintConfiguration {
             } else {
                 // Otherwise we apply the updates on top of the existing select_set.
                 for (rule, enabled) in select_map_updates {
-                    if enabled {
-                        select_set.insert(rule);
-                    } else {
-                        select_set.remove(rule);
-                    }
+                    select_set.set(rule, enabled);
                 }
 
                 for rule in docstring_override_updates {
@@ -879,11 +890,7 @@ impl LintConfiguration {
                 }
             } else {
                 for (rule, enabled) in fixable_map_updates {
-                    if enabled {
-                        fixable_set.insert(rule);
-                    } else {
-                        fixable_set.remove(rule);
-                    }
+                    fixable_set.set(rule, enabled);
                 }
             }
 
@@ -1080,31 +1087,27 @@ impl LintConfiguration {
 
     #[must_use]
     pub fn combine(self, config: Self) -> Self {
+        let mut rule_selections = config.rule_selections;
+        rule_selections.extend(self.rule_selections);
+
+        let mut extend_safe_fixes = config.extend_safe_fixes;
+        extend_safe_fixes.extend(self.extend_safe_fixes);
+
+        let mut extend_unsafe_fixes = config.extend_unsafe_fixes;
+        extend_unsafe_fixes.extend(self.extend_unsafe_fixes);
+
+        let mut extend_per_file_ignores = config.extend_per_file_ignores;
+        extend_per_file_ignores.extend(self.extend_per_file_ignores);
+
         Self {
             exclude: self.exclude.or(config.exclude),
             preview: self.preview.or(config.preview),
-            rule_selections: config
-                .rule_selections
-                .into_iter()
-                .chain(self.rule_selections)
-                .collect(),
-            extend_safe_fixes: config
-                .extend_safe_fixes
-                .into_iter()
-                .chain(self.extend_safe_fixes)
-                .collect(),
-            extend_unsafe_fixes: config
-                .extend_unsafe_fixes
-                .into_iter()
-                .chain(self.extend_unsafe_fixes)
-                .collect(),
+            rule_selections,
+            extend_safe_fixes,
+            extend_unsafe_fixes,
             allowed_confusables: self.allowed_confusables.or(config.allowed_confusables),
             dummy_variable_rgx: self.dummy_variable_rgx.or(config.dummy_variable_rgx),
-            extend_per_file_ignores: config
-                .extend_per_file_ignores
-                .into_iter()
-                .chain(self.extend_per_file_ignores)
-                .collect(),
+            extend_per_file_ignores,
             external: self.external.or(config.external),
             ignore_init_module_imports: self
                 .ignore_init_module_imports
@@ -1119,6 +1122,7 @@ impl LintConfiguration {
             // Plugins
             flake8_annotations: self.flake8_annotations.combine(config.flake8_annotations),
             flake8_bandit: self.flake8_bandit.combine(config.flake8_bandit),
+            flake8_boolean_trap: self.flake8_boolean_trap.combine(config.flake8_boolean_trap),
             flake8_bugbear: self.flake8_bugbear.combine(config.flake8_bugbear),
             flake8_builtins: self.flake8_builtins.combine(config.flake8_builtins),
             flake8_comprehensions: self
@@ -1313,6 +1317,7 @@ fn warn_about_deprecated_top_level_lint_options(
         used_options.push("extend-unsafe-fixes");
     }
 
+    #[allow(deprecated)]
     if top_level_options.ignore_init_module_imports.is_some() {
         used_options.push("ignore-init-module-imports");
     }
@@ -1349,6 +1354,10 @@ fn warn_about_deprecated_top_level_lint_options(
         used_options.push("flake8-bandit");
     }
 
+    if top_level_options.flake8_boolean_trap.is_some() {
+        used_options.push("flake8-boolean-trap");
+    }
+
     if top_level_options.flake8_bugbear.is_some() {
         used_options.push("flake8-bugbear");
     }
@@ -1467,15 +1476,18 @@ fn warn_about_deprecated_top_level_lint_options(
 
 #[cfg(test)]
 mod tests {
-    use crate::configuration::{LintConfiguration, RuleSelection};
-    use crate::options::PydocstyleOptions;
+    use std::str::FromStr;
+
     use anyhow::Result;
+
     use ruff_linter::codes::{Flake8Copyright, Pycodestyle, Refurb};
     use ruff_linter::registry::{Linter, Rule, RuleSet};
     use ruff_linter::rule_selector::PreviewOptions;
     use ruff_linter::settings::types::PreviewMode;
     use ruff_linter::RuleSelector;
-    use std::str::FromStr;
+
+    use crate::configuration::{LintConfiguration, RuleSelection};
+    use crate::options::PydocstyleOptions;
 
     const PREVIEW_RULES: &[Rule] = &[
         Rule::IsinstanceTypeNone,
